#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS

# This software is published by the license of CPU-OS License, you can use and 
# distibuted this software under this License. See CPU-OS Liense for more detail.

# You should have receive a copy of CPU-OS License. If not, please contact us 
# by email <support_os@cpu-os.ac.cn>  

import os,sys
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk,GObject
import threading
import time
import urllib2
import traceback
import zipfile
import shutil
import json
import gnupg,hashlib
import subprocess
import fcntl
import commands
from random import Random
import logging
import platform

patchmgr_app_path = "/usr/share/cdos-update/"
patches_dir = patchmgr_app_path + "/updates"
patches_tmp_dir = "/tmp/cdosupdate"

class Status:
    INITED = 0
    DOWNLOADING = 10
    DOWNLOADED = 14
    ALLDOWNLOADED = 15
    UPDATING = 20
    VERIFYING = 22
    INSTALLING = 23
    INSTALLED = 25
    ALLINSTALLED = 30
    ERROR_INSTALL = 51

def random_str(randomlength=8):
    str = ''
    chars = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789'
    length = len(chars) - 1
    random = Random()
    for i in range(randomlength):
        str+=chars[random.randint(0, length)]
    return str

class Result:
    __error = None
    __value = None

    def __init__(self, error, value):
        self.__error = error
        self.__value = value

    def getError(self):
        return self.__error

    def getValue(self):
        return self.__value

class UIHandler:
    __installHander = None

    def __init__(self, builder, handler):
        self.__installHander = handler

    def get_progress(self, strText, patchID, pkNo, pksum, progressNo, progressSum):
        return (pkNo + progressNo * 1.0 / progressSum )* 100.0 / pksum

    def set_patch_progress(self, strText, patchID, pkNo, pksum, progressNo, progressSum):
        if pksum == 0:
            self.set_progress( strText, 1)
        else:
            self.set_progress( strText + ' (' + str(pkNo + 1) + "/" + str(pksum) + ")" + patchID, (pkNo + progressNo * 1.0 / progressSum )* 100.0 / pksum)

    def set_error_progress(self, errorText, patchID, pkNo, pksum):
            self.set_progress( errorText + ': '  + patchID, -1)

    def set_progress(self, strText, progress_value):
        logging.info(strText)

class InstallHandler:
    __isclose = False
    __uihandler = None
    __updateThread = None
    __autoRestartAfterFinished = False

    def __init__(self):
        self.__isclose = False
        self.__uihandler = UIHandler(None, self)

    def start_Update(self, getpatch_site_path, patchIDList, progressCallback, onlyDownload=False):
        logging.info("Start update ...");
        if self.__updateThread == None or self.__updateThread.is_alive() == False:
            self.__updateThread = threading.Thread(target=self.__updatePatches, args=(getpatch_site_path, patchIDList, progressCallback, onlyDownload, ))
            self.__updateThread.start()
            return True
        else:
            return False

    def __updatePatches(self, getpatch_site_path, patchIDList, progressCallback, onlyDownload=False):
        patchnumber = len(patchIDList)
        correct_install_patches_list = []
        i = 0
        toInstallPatchID = []
        toInstallPatchFile = []
        toInstallPatchNumber = 0
        reboot_flag = False
        # 0 初始化系统目录和临时目录
        res = self.__init_dir()
        if res.getError()!=None:
            logging.error('Error happened when init patch dir!: %s' % res.getError())
            progressCallback(Status.ERROR_INSTALL, "目录初始化失败:", "", correct_install_patches_list, patchIDList, 0)
            return
        patchID = ""
        patchFileList = []
        i = 0
        downloadProgressSum = 50
        if onlyDownload:
            downloadProgressSum = 100
        while self.__isclose==False and i < patchnumber:
            patchID = patchIDList[i]
            # 检查是否已安装此补丁
            progressCallback(Status.DOWNLOADING, '正在下载更新', patchID, correct_install_patches_list, patchIDList, (i + 0.5)  * downloadProgressSum / patchnumber)
            if os.path.exists(patches_dir + "/" +patchID + ".cdospatch"):
                logging.info(patchID + " 已经安装 , 不需要下载")
                i += 1
                continue
            # 检查是否已下载此补丁
            patchfilepath = patches_tmp_dir + "/" +patchID + ".cdospatch"
            if os.path.exists(patchfilepath):
                logging.info(patchID + " 已经下载 , 不需要重复下载")
                toInstallPatchID.append(patchID)
                toInstallPatchFile.append(patchfilepath)
                i += 1
                continue
            # 1 download patch 返回patch文件完整路径
            logging.info("正在下载 " + patchID + "...")
            res = self.__download_patch(getpatch_site_path, patchID, patchfilepath)
            if res.getError()!=None:
                logging.error('Error happened when download patch package! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "下载更新失败: " + res.getError(), patchID, correct_install_patches_list, patchIDList, 0)
                return
            toInstallPatchID.append(patchID)
            toInstallPatchFile.append(patchfilepath)
            progressCallback(Status.DOWNLOADED, '下载更新', patchID, correct_install_patches_list, patchIDList, (i + 1)  * downloadProgressSum / patchnumber)
            i += 1
        if onlyDownload == True:
            progressCallback(Status.ALLDOWNLOADED, '所有补丁均已下载', "", correct_install_patches_list, patchIDList, 100)
            return
        logging.info("正在安装的补丁：" + str(toInstallPatchID))
        toInstallPatchNumber = len(toInstallPatchID)
        i = 0
        while self.__isclose==False and i < toInstallPatchNumber:
            patchID = toInstallPatchID[i]
            patchfilepath = toInstallPatchFile[i]
            logging.info("正在安装 " + patchID + "...")
            # 2 初始化patch解压目录
            progressCallback(Status.UPDATING, '正在初始化更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.1)  * 50.0 / toInstallPatchNumber)
            res = self.__init_patch_dir(patchID)
            if res.getError()!=None:
                logging.error('Error happened when init patch dir! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "目录初始化失败:", patchID, correct_install_patches_list, toInstallPatchID, 0)
                return
            patchfiledir = res.getValue()
            # 3 解压patch文件
            progressCallback(Status.VERIFYING, '正在解压更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.2)  * 50.0 / toInstallPatchNumber)
            res = self.__unzip_patch(patchfilepath, patchfiledir)
            if res.getError()!=None:
                logging.error('Error happened when install patch package! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "解压更新失败: " + res.getError(), patchID, correct_install_patches_list, toInstallPatchID, 0)
                shutil.rmtree(patchfiledir)
                return
            # 4 校验patch内容
            progressCallback(Status.VERIFYING, '正在校验更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.4)  * 50.0 / toInstallPatchNumber)
            res = self.__check_patch(patchfiledir)
            if res.getError()!=None:
                logging.warning('Warning happened when __check_patch patch package! :%s' % res.getError())
                shutil.rmtree(patchfiledir)
                i += 1
                continue
            patchinfo = res.getValue()
            if not patchID == patchinfo['id']:
                logging.error('Error happened when check patch id! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "校验更新未通过: ID不匹配", patchID, correct_install_patches_list, toInstallPatchID, 0)
                shutil.rmtree(patchfiledir)
                return
            # 检查是否已安装此补丁
            if os.path.exists(patches_dir + "/" +patchID + ".cdospatch"):
                logging.error(patchID + ' has been installed! :%s' % res.getError())
                shutil.rmtree(patchfiledir)
                i += 1
                continue
            # 5 安装patch内容：替换文件
            progressCallback(Status.INSTALLING, '正在安装更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.6)  * 50.0 / toInstallPatchNumber)
            res = self.__installfile_patch(patchinfo, patchfiledir)
            if res.getError()!=None:
                logging.error('Error happened when __installfile_patch patch package! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "安装更新失败: " + res.getError(), patchID, correct_install_patches_list, toInstallPatchID, 0)
                shutil.rmtree(patchfiledir)
                return
            # 6 安装patch内容：执行命令
            progressCallback(Status.INSTALLING, '正在安装更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.8)  * 50.0 / toInstallPatchNumber)
            res = self.__installcommand_patch(patchfiledir)
            if res.getError()!=None:
                logging.error('Error happened when __installcommand_patch patch package! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "安装更新失败: " + res.getError(), patchID, correct_install_patches_list, toInstallPatchID, 0)
                #shutil.rmtree(patchfiledir)
                return
            # 7 清理临时目录：执行命令
            progressCallback(Status.INSTALLING, '正在安装更新', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 0.9)  * 50.0 / toInstallPatchNumber)
            res = self.__installfinished_patch(patchID, patchfilepath, patchfiledir)
            if res.getError()!=None:
                logging.error('Error happened when __installfinished_patch patch package! :%s' % res.getError())
                progressCallback(Status.ERROR_INSTALL, "安装更新失败: " + res.getError(), patchID, correct_install_patches_list, toInstallPatchID, 0)
                return
            if patchinfo['reboot'] == 'true':
                reboot_flag = True
            correct_install_patches_list.append(patchID)
            progressCallback(Status.INSTALLED, '更新安装完毕', patchID, correct_install_patches_list, toInstallPatchID, 50 + (i + 1)  * 50.0 / toInstallPatchNumber)
            i += 1
        logging.debug("是否重启:" + str(self.__autoRestartAfterFinished))
        if self.__isclose==True:
            self.__exit()
        if  i == 0:
            progressCallback(Status.ALLINSTALLED, '已完成更新', "", correct_install_patches_list, patchIDList, 100)
        else:
            param_list = {'reboot':reboot_flag}
            if i == toInstallPatchNumber:
                logging.info("correct_install_patches_list:" + str(correct_install_patches_list))
                progressCallback(Status.ALLINSTALLED, '成功安装更新', "", correct_install_patches_list, patchIDList, 100, paramList=param_list)
            else:
                #理论上不会进入该段代码

                progressCallback(Status.ALLINSTALLED, '所有补丁均已安装', "", correct_install_patches_list, patchIDList, 100, paramList=param_list)

    def __init_dir(self):
        try:
            if not os.path.exists(patches_dir):
                os.makedirs(patches_dir, 0644)
            if not os.path.exists(patches_tmp_dir):
                os.makedirs(patches_tmp_dir, 0777)
                os.chmod(patches_tmp_dir, 0777)
            return Result(None, None)
        except OSError as e1:
            logging.error('Exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('Exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('Exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __init_patch_dir(self, patchID):
        try:
            patch_tmp_dir = patches_tmp_dir + "/" + patchID + "_" + random_str(6)
            if os.path.exists(patch_tmp_dir):
                shutil.rmtree(patch_tmp_dir)
            os.makedirs(patch_tmp_dir, 0777)
            os.chmod(patch_tmp_dir, 0777)
            return Result(None, patch_tmp_dir)
        except OSError as e1:
            logging.error('Exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('Exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('Exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __download_patch(self, getpatch_site_path, patchID, patchfilePath):
        patch_file = patchfilePath
        downloading_file = patch_file + ".cdospatch"
        try:
            if os.path.exists(patch_file):
                os.remove(patch_file)
            if os.path.exists(downloading_file):
                os.remove(downloading_file)
            url = getpatch_site_path + patchID + ".cdospatch"

            u = urllib2.urlopen(url, timeout=10)
            f = open(downloading_file, 'wb')
            meta = u.info()
            file_size = int(meta.getheaders("Content-Length")[0])
            logging.debug("Downloading: %s Bytes: %s" % (patch_file, file_size))

            file_size_dl = 0
            block_sz = 8192
            while True:
                buffer = u.read(block_sz)
                if not buffer:
                    break

                file_size_dl += len(buffer)
                f.write(buffer)
                status = r"%10d  [%3.2f%%]" % (file_size_dl, file_size_dl * 100. / file_size)
                status = status + chr(8)*(len(status)+1)

            f.close()
            os.rename(downloading_file, patch_file)
            time.sleep(1)
            return Result(None, patch_file)
        except urllib2.HTTPError as e1:
            logging.error('HTTPError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("请检查网络", None)
        except urllib2.URLError as e2:
            logging.error('URLError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("请检查网络", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请检查网络", None)

    def __unzip_patch(self, patchfilepath, patchfiledir):
        try:
            zfile = zipfile.ZipFile(patchfilepath)
            zfile.extractall(patchfiledir)
            return Result(None, patchfiledir)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __check_patch(self, patchfiledir):
        try:
            if os.path.exists(patchfiledir + "/info.json.sign") and os.path.exists(patchfiledir + "/info.json"):
                gpg_res = self.__gpg_check_patch(patchfiledir)
                if gpg_res.getError() != None:  #有错误，返回错误
                    return gpg_res

                f = file(patchfiledir + "/info.json")
                patchinfo = json.load(f)
                f.close()
                file_res = self.__hash_check(patchfiledir + '/files', patchinfo['files'])
                if file_res.getError() != None:  #有错误
                    return file_res  #返回错误的信息
                command_res = self.__hash_check(patchfiledir + '/command', patchinfo['command'])
                if command_res.getError() != None:  #有错误
                    return command_res  #返回错误的信息
                sys_res = self.__sysVersion_check(patchinfo["systemversion"])
                if sys_res.getError() != None:
                    return sys_res
                package_res = self.__check_package(patchinfo["package"])
                if package_res.getError() != None:
                    return package_res
                return Result(None, patchinfo)  #返回结果
            return Result("更新校验失败，补丁文件被篡改，请检查更新服务器配置", None)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __installfile_patch(self, patchinfo, patchfiledir):
        try:
            files_dir = patchfiledir + "/files"
            files_dir_len = len(files_dir)
            for root, dirs, files in os.walk(files_dir):
                for name in files:
                    orifile = root + "/" + name
                    objdir = root[files_dir_len:]
                    objfilename = name
                    objfilepath = objdir + "/" + objfilename
                    if not os.path.isdir(objdir):
                        os.makedirs(objdir)
                    shutil.copy2(orifile, objfilepath)
            time.sleep(1)
            return Result(None, None)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __installcommand_patch(self, patchfiledir):
        try:
            files_dir = patchfiledir + "/command"
            files_dir_len = len(files_dir)
            for root, dirs, files in os.walk(files_dir):
                files.sort()
                for name in files:
                    objfile = root + "/" + name
                    objdir = root[files_dir_len:]
                    objfilename = name
                    objfilepath = objdir + "/" + objfilename
                    logging.info("Executing command:" + objfile)
                    call_res = subprocess.call(["bash", objfile])
                    if not (call_res == 0):
                        return Result("补丁脚本执行错误", None)
            return Result(None, None)
        except Exception as e:
            logging.error('generic exception %s:  %s' % (e, traceback.format_exc()))
            return Result("补丁更新异常，请检查：", None)

    def __installfinished_patch(self, patchID, patchfilepath, patchfiledir):
        try:
            shutil.move(patchfilepath, patches_dir + "/" +patchID + ".cdospatch")
            shutil.rmtree(patchfiledir)
            return Result(None, None)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    def __exit(self):
        if self.__autoRestartAfterFinished == True:
            logging.debug("TODO: 执行重启")
        Gtk.main_quit()

    @staticmethod
    def  __gpg_check_patch( patchfiledir):
        try:
            keypath = sys.path[0] + '/keyring'
            gpg = gnupg.GPG( gnupghome = keypath)
            verify = gpg.verify_file( open(patchfiledir + "/info.json.sign", "rb"), patchfiledir + "/info.json")
            #gpg校验之后删除多余文件
            if os.path.exists(keypath + '/secring.gpg'):
                sec_del_res = MyFile.file_delete(keypath + '/secring.gpg')
                if sec_del_res.getError() != None:
                    return Result('secring.gpg删除错误',None)
            if os.path.exists(keypath + '/trustdb.gpg'):
                tru_del_res = MyFile.file_delete(keypath + '/trustdb.gpg')
                if tru_del_res.getError() != None:
                    return Result('trustdb.gpg删除错误',None)
            #对gpg结果进行判断
            if 'valid' in verify.__dict__ :
                if verify.valid :
                    return Result(None, "gpg校验结束，info文件与签名匹配")
                else:
                    return Result("gpg校验结束，info文件与签名不匹配",None)
            else:
                return Result("gpg check error",None)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    @staticmethod
    def __hash_check( patchfiledir, patchinfo):
        try:
            if (len(patchinfo) == 0) and (not os.path.exists(patchfiledir)):
                return Result(None, None)
            patch_file_obj = MyFile(patchfiledir)
            relativefilelist = patch_file_obj.getRelativeFile()
            paths = []
            info_json = {}
            for file_id in range(len(patchinfo) ):
                paths.append( patchinfo[file_id]["path"] )
                #获取并整合基本信息
                info_json[ patchinfo[file_id]["path"] ] = { 'type': patchinfo[file_id]['hash']["type"], 'value': patchinfo[file_id]['hash']["value"]}
            if len(relativefilelist) == len(paths):
                for i in range( len(relativefilelist)):
                    if (relativefilelist[i] not in paths):
                        return Result('文件路径不匹配', None)
                logging.debug("文件路径校验成功")
            else:
                #文件路径不匹配
                return Result( "文件路径不匹配",None)
            logging.debug("文件内容校验")
            filelist = patch_file_obj.getfile()
            number = 0
            for patch_file in filelist:
                hashtype = info_json[ patch_file[len(patchfiledir):] ]['type']
                hash_res = MyFile.getHashByFile(patch_file, hashtype)
                if hash_res.getError() != None:  #hash过程出错
                    return hash_res
                elif hash_res.getValue() != info_json[ patch_file[len(patchfiledir):] ]['value']:  #hash过程没有问题，且文件校验不匹配
                    return Result("文件内容校验失败:hash不匹配", None)
            logging.debug("文件内容校验成功")
            return Result(None,"文件校验成功")
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    @staticmethod
    def __sysVersion_check( patchinfo):
        try:
            # 如果该补丁没有版本要求
            if ( (type(patchinfo)==list) and (len(patchinfo) == 0) ) or (patchinfo == None) or (patchinfo == "null"):
                return Result(None, True)
            if type(patchinfo) != list:
                return Result('系统版本信息类型错误', None)
            (sys_id, sys_release, sys_codename) = platform.linux_distribution()
            # 这里为获取系统ID,与lsb_release -a中ID相同,但是当前由于CDOS本身bug，所以这里得到的不是CDOS，先注释
            # if not (sys_id == "CDOS"):
            #     return Result("该操作系统不为CDOS，请安装CDOS", None)
            if sys_release.split('-')[0] not in patchinfo:
                return Result("补丁不支持该系统版本", None)
            return Result(None, True)
        except Exception as ee:
            logging.error('generic exception %s:  %s' % (ee, traceback.format_exc()))
            return Result("系统版本校验异常", None)

    @staticmethod
    def __check_package(patchinfo):
        try:
            # 如果没有包需要检测
            if ( (type(patchinfo) == dict) and (len(patchinfo) == 0) ) or (patchinfo == None) or (patchinfo == "null"):
                return Result(None, True)
            if type(patchinfo) != dict:
                return Result('包版本信息类型错误', None)
            # 获取本地已经正确安装的deb包及版本的json描述(即ii状态的deb包)
            deblist = subprocess.check_output(["dpkg","-l"]).split('\n')
            if len(deblist) <= 5:
                return Result('没有deb列表', None)
            deb_desc_json = {}
            for i in range(5,len(deblist) - 1):
                deb_desc = deblist[i].split()
                if (len(deb_desc) > 0) and (deb_desc[0] == "ii"):
                    deb_desc_json[deb_desc[1]] = deb_desc[2]
            # 开始校验包版本
            for deb_name in patchinfo:
                # 如果本地deb包中没有该包
                if deb_name not in deb_desc_json:
                    return Result('包版本校验失败', None)
                # if type(patchinfo[deb_name]) != list:
                #     pass
                # 如果本地deb包版本不在patchinfo版本列表中
                if type(patchinfo[deb_name]) == list and len(patchinfo[deb_name]) != 0 and deb_desc_json[deb_name] not in patchinfo[deb_name]:
                    return Result('包版本校验失败', None)
            return Result(None, True)
        except Exception as ee:
            logging.error('generic exception %s:  %s' % (ee, traceback.format_exc()))
            return Result('包版本校验异常', None)


class MyFile:
    def __init__(self, parentdir):
        self.__filelist = []
        self.__parentdir = parentdir
#遍历目录,得到文件
    def searchfile(self, Pdir):
        for list_dirs in os.listdir(Pdir):
            path = os.path.join(Pdir, list_dirs)
            #如果是文件
            if os.path.isfile(path):
                self.__filelist.append(path)
            #如果是目录,遍历该目录
            if os.path.isdir(path):
                self.searchfile(path)
#获取返回的文件列表
    def getfile(self):
        self.__filelist = []
        self.searchfile(self.__parentdir)
        return self.__filelist

    def getRelativeFile(self):
        self.__filelist = []
        self.searchfile(self.__parentdir)
        list = []
        for filename in self.__filelist:
            if self.__parentdir[-1] != "/":
                list.append(filename[len(self.__parentdir):])
            else:
                list.append('/'+ filename[len(self.__parentdir):])
        return list

    #静态函数
    @staticmethod
    def getHashByFile(filename, type):
        try:
            BLOCKSIZE = 65536
            if (type == "sha224") or (type == "sha256") or (type == "sha384") or (type == "sha512"):
                hasher = hashlib.new(type)
            with open(filename, 'rb') as file_obj:
                buf = file_obj.read(BLOCKSIZE)
                while len(buf) > 0:
                    hasher.update(buf)
                    buf = file_obj.read(BLOCKSIZE)
            return Result(None, hasher.hexdigest())
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("未知异常，请稍后重试", None)

    #文件删除，如果文件/文件夹存在，进行对应删除工作
    @staticmethod
    def file_delete(filename):
        try:
            if os.path.exists(filename):
                if os.path.isfile(filename):
                    os.remove(filename)
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
            return Result(None, None)
        except OSError as e1:
            logging.error('OSError exception %s:  %s' % (e1, traceback.format_exc()))
            return Result("file_delete:权限不足，需要管理员权限", None)
        except IOError as e2:
            logging.error('IOError exception %s:  %s' % (e2, traceback.format_exc()))
            return Result("file_delete:文件处理异常，请检查", None)
        except Exception as e3:
            logging.error('generic exception %s:  %s' % (e3, traceback.format_exc()))
            return Result("file_delete:未知异常，请稍后重试", None)
